/***************************************************************************
 *   Copyright (C) 2008 by Daniel Calviño Sánchez                          *
 *   danxuliu@gmail.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; If not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef KTUTORIAL_H
#define KTUTORIAL_H

#include <kxmlguiwindow.h>

#include "ktutorial_export.h"

#include "TutorialManager.h"

class KAction;

/**
 * Main class of KTutorial.
 * This class is used to setup KTutorial, register new Tutorials embedded in the
 * application and find objects in the Tutorials. It uses a Singleton design
 * pattern.
 *
 * When using KTutorial, KTutorial::setup(KxmlGuiWindow*) must be called before
 * any other KTutorial method. This will set up everything needed for KTutorial
 * to work. It needs to know the main window of the application.
 *
 * Once KTutorial is set up, Tutorials embedded in the application can be added.
 * Just use KTutorial::registerTutorial(Tutorial*) so it is registered in the
 * system. Only registered tutorial are seen by the user.
 *
 * Finally, any of the children objects of the main window of the application
 * (which will usually be at least all widgets) can be got with
 * KTutorial::findObject(const QString&), provided the looked for object has its
 * object name set.
 *
 * The TutorialManagerDialog is shown as modal through the "tutorials" action in
 * Help menu. This action will be automatically disabled whenever a tutorial is
 * being executed.
 */
class KTUTORIAL_EXPORT KTutorial: public QObject {
Q_OBJECT
public:

    /**
     * Returns the only instance of this class.
     *
     * @return The only instance of this class.
     */
    static KTutorial* self() {
        return sSelf;
    }

    /**
     * Sets up everything for KTutorial to work.
     * This is the first method that must be called when using KTutorial in an
     * application.
     *
     * It adds the Tutorials action and the menu entry for it in Help menu.
     *
     * @param window The main window of the application.
     */
    void setup(KXmlGuiWindow* window);

    /**
     * Registers the Tutorial.
     * Only Tutorials with a identifier not added yet can be added. If the
     * Tutorial couldn't be added, false is returned.
     *
     * The Tutorial is reparented and thus deleted when needed. The
     * TutorialInformation of the Tutorial is also deleted automatically.
     *
     * @param tutorial The Tutorial to register.
     * @return True if the Tutorial was registered, false otherwise.
     */
    bool registerTutorial(Tutorial* tutorial) {
        return mTutorialmanager->registerTutorial(tutorial);
    }

    /**
     * Returns the parent widget.
     * It is the main window of the application.
     *
     * @return The parent widget.
     */
    KXmlGuiWindow* parentWidget() const {
        return mParent;
    }

    /**
     * Returns the object with the specified name, if any.
     * Objects are searched in the children of the main window of the
     * application.
     *
     * @param name The name of the object to find.
     * @return The object with the specified name, or null if there is none.
     * @todo Is it needed to look in other parent objects apart of the main
     *       window?
     */
    template <typename T>
    T findObject(const QString& name) const {
        return mParent->findChild<T>(name);
    }

public slots:

    /**
     * Shows a modal TutorialManagerDialog.
     * Called when the tutorials action is triggered.
     */
    void showTutorialManagerDialog() const;

private:

    /**
     * The instance of this class.
     */
    static KTutorial* sSelf;

    /**
     * The manager for the tutorials.
     */
    TutorialManager* mTutorialmanager;

    /**
     * The parent widget of the TutorialManagerDialog to show.
     */
    KXmlGuiWindow* mParent;

    /**
     * The KAction to show the TutorialManagerDialog.
     */
    KAction* mTutorialsAction;

    /**
     * Creates a new KTutorial.
     * Private to avoid classes other than self to create instances.
     */
    KTutorial(): QObject(),
        mTutorialmanager(new TutorialManager()) {
        mTutorialmanager->setParent(this);
        mTutorialsAction = 0;
    }

private slots:

    /**
     * Disables mTutorialsAction.
     * Just a wrapper to be connected with signals that can't pass false to the
     * setEnabled(bool) slot.
     */
    void disableTutorialsAction();

    /**
     * Enables mTutorialsAction.
     * Just a wrapper to be connected with signals that can't pass true to the 
     * setEnabled(bool) slot.
     */
    void enableTutorialsAction();

};

#endif
